#pragma once

#include <vector>
#include <string>
#include <memory>
#include <stack>

namespace groove {

	class Buffer {
	private:
		std::vector<std::string> lines;
		std::vector<std::string> reverted;
		std::stack<std::vector<std::string>> history;
		void removeTabs(std::string& line) {
			ulong tab = line.find("\t");
			if(tab != line.npos)
				this->removeTabs(line.replace(tab, 1, "    "));
		}
	public:
		bool changed;
		Buffer() : lines(std::vector<std::string>()), reverted(std::vector<std::string>()), changed(false) {}
		std::vector<std::string>& linebuffer() {
			return this->lines;
		}

		void insert(std::string str, unsigned long line) {
			this->lines.insert(this->lines.begin() + line, str);
			this->removeTabs(this->lines.at(line));
			this->changed = true;
		}

		void insert(std::string line) {
			this->removeTabs(line);
			this->lines.emplace_back(line);
			this->changed = true;
		}
		void insert(unsigned long after) {
			this->lines.insert(this->lines.begin() + after + 1, "");
			this->changed = true;
		}

		void remove(unsigned long line) {
			this->lines.erase(this->lines.begin() + line);
			this->changed = true;
		}

		void remove(unsigned long line, int car) {
			this->lines.at(line).erase(this->lines.at(line).begin() + car);
			this->changed = true;
		}

		void deleteChar(unsigned long line, int car) {
			this->lines.at(line).erase(this->lines.at(line).begin() + car);
			this->changed = true;
		}

		std::string& at(unsigned long line) {
			return this->lines.at(line);
		}

		void swap(std::vector<std::string> buffer) {
			this->history.push(this->lines);
			this->lines    = buffer;
		}

		void revert() {
			this->lines = this->history.top();
			this->history.pop();
		}

		unsigned long size() {
			return this->lines.size();
		}
	};

}