#pragma once

#include <functional>
#include <memory>
#include <vector>

namespace groove {

	typedef std::function<void()> lambda;

	struct HistoryNode {
		lambda undo, redo;
		HistoryNode(lambda undo, lambda redo) :
				undo(std::move(undo)), redo(std::move(redo)) {}
	};

	class History {
	private:
		std::unique_ptr<std::vector<HistoryNode>> history;
		unsigned long pointer;

		void cutTail() {
			if (this->history->begin() == this->history->end() || this->pointer >= this->history->size())
				return;
			this->history->erase(this->history->begin() + this->pointer + 1, this->history->end());
		}
	public:
		History() : history(std::make_unique<std::vector<HistoryNode>>()), pointer(0) {
			this->push(HistoryNode([](){}, [](){}));
		}
		void push(HistoryNode node) {
			this->cutTail();
			this->history->emplace_back(std::move(node));
			this->pointer = this->history->size() - 1;
		}

		void undo() {
			if (this->pointer <= 0)
				return;
			this->history->at(this->pointer).undo();
			this->pointer--;
		}

		void redo() {
			if (this->pointer + 1 >= this->history->size())
				return;
			this->history->at(this->pointer + 1).redo();
			this->pointer++;
		}
	};

}