#include <config/ConfigParser.hpp>
#include <fstream>

namespace groove {
	namespace config {

		void ConfigValue::set(std::string &value) {
			strVal  = value;
			boolVal = value == "true";
			try {
				intVal = std::stoi(value);
			} catch (std::invalid_argument) {
				intVal = 0;
			}
			try {
				floatVal = std::stof(value);
			} catch (std::invalid_argument) {
				floatVal = 0.0f;
			}
		}

		void ConfigParser::parse() {
			std::ifstream file(this->filename.c_str());
			if(file.is_open()) {
				while(!file.eof()) {
					std::string line;
					std::getline(file, line);
					if (line == "" || line.at(0) == '#')
						continue;
					std::vector<std::string> pair = std::explode(line,'=');
					std::string key = pair.at(0);
					std::string value = pair.at(1);
					std::trim(key);
					std::trim(value);
					this->params->emplace(key, ConfigValue(value));
				}
			}
			else {
				std::cerr << "Could not load config file." << std::endl;
			}
		}

		template<>
		std::string ConfigParser::get<std::string>(const std::string& key) {
			if (this->params->find(key) == this->params->end())
				return "";
			return this->params->at(key).strVal;
		}

		template<>
		int ConfigParser::get<int>(const std::string& key) {
			if (this->params->find(key) == this->params->end())
				return 0;
			return this->params->at(key).intVal;
		}

		template<>
		float ConfigParser::get<float>(const std::string& key) {
			if (this->params->find(key) == this->params->end())
				return 0.0f;
			return this->params->at(key).floatVal;
		}

		template<>
		bool ConfigParser::get<bool>(const std::string& key) {
			if (this->params->find(key) == this->params->end())
				return false;
			return this->params->at(key).boolVal;
		}

	}
}