#pragma once

#include <string>
#include <unordered_map>
#include <vector>
#include <sstream>
#include <algorithm>
#include <iostream>
#include <pwd.h>
#include <unistd.h>
#include <sys/types.h>
#include <memory>

namespace std {
	static inline std::vector<std::string> explode(const std::string &str, char delimiter) {
		std::vector<std::string> tokens;
		std::stringstream inStream(str);
		std::string tempString;

		while (std::getline(inStream, tempString, delimiter))
			tokens.push_back(tempString);

		return tokens;
	}

	static inline void ltrim(std::string &string) {
		string.erase(string.begin(), std::find_if(string.begin(), string.end(), [](auto c){return !std::isspace(c);}));
	}

	static inline void rtrim(std::string &string) {
		string.erase(std::find_if(string.rbegin(), string.rend(), [](auto c){return !std::isspace(c);}).base(), string.end());
	}

	static inline void trim(std::string &string) {
		rtrim(string);
		ltrim(string);
	}
}

namespace groove {
	namespace config {

		struct ConfigValue {
			std::string strVal;
			int intVal;
			float floatVal;
			bool boolVal;

			ConfigValue(std::string &value) {
				set(value);
			}

			void set(std::string &value);
		};

		class ConfigParser {
		private:
			std::unique_ptr<std::unordered_map<std::string, ConfigValue>> params;
			std::string filename;

			void parse();
		public:
			ConfigParser() : params(std::make_unique<std::unordered_map<std::string, ConfigValue>>()) {
				this->filename = getenv("HOME");
				this->filename += "/.grooverc";
				this->parse();
			}
			ConfigParser(const ConfigParser&) = delete;

			template <typename T>
			T get(const std::string& key);

			template<typename T>
			void set(std::string key, T value) {
				std::string newValue = std::to_string(value);
				this->params->at(key).set(newValue);
			}
		};

	}
}