#include <modes/Edit.hpp>
#include <Editor.hpp>

namespace groove {
	namespace modes {

		void Edit::input(int c) {
			this->status_ = this->editor.buffer->changed ? "*" : "";
			if (this->editor.movement(c))
				return;
			switch (c) {
				case 'q':
					this->editor.mode_ = groove::Mode::EXIT;
					this->editor.input(' ');
					break;
				case 'i':
					{
						std::vector<std::string>& oldBuffer = this->editor.buffer->linebuffer();
						this->editor.history.push(HistoryNode(
								[this, oldBuffer](){
									this->editor.buffer->swap(oldBuffer);
								}, [this](){
									this->editor.buffer->revert();
								})
						);
						this->editor.mode_ = groove::Mode::INSERT;
					}
					break;
				case 'a':
					this->editor.x = static_cast<int>(this->editor.buffer->at(this->editor.y).length());
					this->editor.mode_ = groove::Mode::INSERT;
					break;
				case 's':
					this->editor.mode_ = groove::Mode::SAVE;
					break;
				case 'c':
					this->editor.clipboard = this->editor.buffer->at(this->editor.y);
					break;
				case 'x':
					{
						long y = this->editor.y;
						std::string line = this->editor.buffer->at(y);
						this->editor.history.push(
								HistoryNode([this, y, line](){
									this->editor.buffer->insert(line, y);
								}, [this, y](){
									this->editor.buffer->remove(y);
								})
						);
						this->editor.clipboard = this->editor.buffer->at(y);
						this->editor.buffer->remove(y);
					}
					break;
				case 'v':
					{
						std::string clipboard = this->editor.clipboard;
						long y = this->editor.y;
						this->editor.history.push(
								HistoryNode([this, y](){
									this->editor.buffer->remove(y);
								},[this, y, clipboard](){
									this->editor.buffer->insert(clipboard, y);
									this->editor.y = y;
									this->editor.x = this->editor.buffer->at(y).size();
								})
						);
						this->editor.buffer->insert(clipboard, y);
						this->editor.y++;
						this->editor.x = this->editor.buffer->at(y).size();
					}
					break;
				case 'd':
					{
						if (this->editor.y >= this->editor.buffer->size() - 1)
							break;
						long y = this->editor.y;
						std::string line = this->editor.buffer->at(y);
						this->editor.history.push(
								HistoryNode([this, y, line](){
									this->editor.buffer->insert(line, y);
								}, [this, y, line](){
									this->editor.buffer->remove(y);
								})
						);
						this->editor.buffer->remove(this->editor.y);
					}
					break;
				case 'l':
					switch (this->editor.lineMode) {
						case LineMode::NUMBERS:
							this->editor.lineMode = LineMode::RELATIVE;
							break;
						case LineMode::RELATIVE:
							this->editor.lineMode = LineMode::NONE;
							break;
						case LineMode::NONE:
							this->editor.lineMode = LineMode::NUMBERS;
							break;
					}
					break;
				case 'f':
					this->editor.mode_ = groove::Mode::SEARCH;
					this->editor.input(' ');
					break;
				case 'u':
					this->editor.history.undo();
					break;
				case 'r':
					this->editor.history.redo();
					break;
				case '^':
					this->editor.x = 0;
					break;
				case '$':
					this->editor.x = this->editor.buffer->at(this->editor.y).size();
					break;
				case 'g':
					this->editor.y = 0;
					this->editor.offset = 0;
					break;
				case 'G':
					this->editor.y = this->editor.buffer->size() - 1;
					this->editor.offset = this->editor.buffer->size() - LINES + 1;
					break;
				case KEY_NPAGE:
					for (int i = 0; i < LINES; i++) {
						this->editor.down();
					}
					break;
				case KEY_PPAGE:
					for (int i = 0; i < LINES; i++) {
						this->editor.up();
					}
					break;
			}
		}
	}
}